{% if notes -%} 
<!-- wp:heading  -->
<h2 id="notes">Notes</h2>
<!-- /wp:heading -->

<!-- wp:list -->

<ul>{% for note in notes %}
<li>{{note | replace('\n', '')}}</li>
{%- endfor %}</ul>

<!-- /wp:list -->

{% endif -%}
