<!-- wp:heading  -->
<h2 id="documentation">Documentation</h2>
<!-- /wp:heading -->

{% for section_type, n in inline_doc_parts -%}
<!-- {{section_type}} -->
{{n}}
<!-- /{{section_type}} -->

{% endfor -%}

{% if doc_links -%}
<!-- wp:heading  {"level":3} -->
<h3 id="documentation_links">Documentation Links</h3>
<!-- /wp:heading -->
<!-- wp:list -->
<ul>{% for title, doc in doc_links.items() %}
<li><a href="{{doc.url}}">{{title}}</a> {{doc.description}}</li>
{%- endfor %}</ul>

<!-- /wp:list -->
{% endif %}