{% if images -%}

<!-- wp:heading -->
<h2>Images</h2>
<!-- /wp:heading -->

<!-- wp:html -->
<ul>{% for title, image in images.items() %}
<li><a href="{{image.url}}" title="{{title}}"><img src="{{image.url}}" width="100"></a>
{{image.description}}</li>
{%- endfor -%}</ul>

<!-- /wp:html -->

{% endif -%}