{# {% include "wordpress/jsonld.html" %} lack of unfiltered_html capability makes this fail --> #}

{% include "wordpress/header.html" %}

{% include "wordpress/resources.html" %}

{% include "wordpress/documentation.html" %}

{% include "wordpress/images.html" %}

{% include "wordpress/contacts.html" %}


{% include "wordpress/schema.html" %}

{% include "wordpress/references.html" %}

{% include "wordpress/footer.html" %}

{% include "wordpress/distributions.html" %}