{% if resources %} 
<!-- wp:heading  -->
<h2 id="resources">Resources</h2>
<!-- /wp:heading -->

<!-- wp:list -->
<ul>{%- for term_name, terms in resources.items() -%}
{%- for resource in terms %}
<li><strong><a href="{{resource.url}}">{{resource.name}}</a></strong>. {{resource.description}}</li>
{%- endfor %}
{%- endfor %}</ul>
<!-- /wp:list -->

{% endif %}