{% if references %} 
<!-- wp:heading -->
<h2 id="references">References</h2>
<!-- /wp:heading -->

<!-- wp:paragraph -->

<p>Urls used in the creation of this data package.</p>
<!-- /wp:paragraph -->

<!-- wp:list -->

<ul>{% for term_name, terms in references.items() -%}
{% for reference in terms %}
{%- if reference.url.startswith('http') -%}
    <li> <strong><a href="{{reference.url}}">{{reference.name}}</a></strong>. {{reference.description}}</li>
{%-else-%}
    <li> <strong>{{reference.url}}</strong>. {{reference.description}}</li>
{%-endif-%}
{%- endfor %}
{%- endfor %}</ul>

<!-- /wp:list -->

{% endif %}